# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class TableGenerationSpec(UniversalBaseModel):
    """
    Specification for which generation to use during table summarization. If not specified, the platform uses the default summarizer. All parameters are optional and overrides the default values.
    """

    llm_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the LLM to use.
    """

    prompt_template: typing.Optional[str] = pydantic.Field(default=None)
    """
    The prompt template to use when generating the table summary. 
    Vectara manages both system and user roles and prompts for the generative
    LLM out of the box by default. However, users can override the
    `prompt_template` via this variable. The `prompt_template` is in the form of an
    Apache Velocity template. For more details on how to configure the
    `prompt_template`, see the [long-form documentation](https://docs.vectara.com/docs/prompts/vectara-prompt-engine).
    """

    model_parameters: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Optional parameters for the specified model used when generating the table summary.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..types.bad_request_error_body import BadRequestErrorBody
from ..types.chat_completion_request_message import ChatCompletionRequestMessage
from ..types.create_chat_completion_response import CreateChatCompletionResponse
from ..types.error import Error

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawLlmClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def chat_completion(
        self,
        *,
        model: str,
        messages: typing.Sequence[ChatCompletionRequestMessage],
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        stream: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateChatCompletionResponse]:
        """
        OpenAI-compatible endpoint for chat completions. Creates a response for the given chat conversation.
        The chat completion API allows you to chat with Vectara's language models in a way that's compatible with OpenAI's specification.
        This makes it easy to integrate with applications already designed for OpenAI's API.

        Parameters
        ----------
        model : str
            The ID of the model to use. This field is required.

        messages : typing.Sequence[ChatCompletionRequestMessage]
            An ordered array of messages that represent the full context of the conversation to date. Each message includes a `role` and `content`.

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        stream : typing.Optional[bool]
            Optional. When set to `true`, the API streams partial message deltas as they become available, similar to ChatGPT's streaming mode.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateChatCompletionResponse]
            A chat completion
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/llms/chat/completions",
            base_url=self._client_wrapper.get_environment().default,
            method="POST",
            json={
                "model": model,
                "messages": convert_and_respect_annotation_metadata(
                    object_=messages, annotation=typing.Sequence[ChatCompletionRequestMessage], direction="write"
                ),
                "stream": stream,
            },
            headers={
                "content-type": "application/json",
                "Request-Timeout": str(request_timeout) if request_timeout is not None else None,
                "Request-Timeout-Millis": str(request_timeout_millis) if request_timeout_millis is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateChatCompletionResponse,
                    parse_obj_as(
                        type_=CreateChatCompletionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        BadRequestErrorBody,
                        parse_obj_as(
                            type_=BadRequestErrorBody,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawLlmClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def chat_completion(
        self,
        *,
        model: str,
        messages: typing.Sequence[ChatCompletionRequestMessage],
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        stream: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateChatCompletionResponse]:
        """
        OpenAI-compatible endpoint for chat completions. Creates a response for the given chat conversation.
        The chat completion API allows you to chat with Vectara's language models in a way that's compatible with OpenAI's specification.
        This makes it easy to integrate with applications already designed for OpenAI's API.

        Parameters
        ----------
        model : str
            The ID of the model to use. This field is required.

        messages : typing.Sequence[ChatCompletionRequestMessage]
            An ordered array of messages that represent the full context of the conversation to date. Each message includes a `role` and `content`.

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        stream : typing.Optional[bool]
            Optional. When set to `true`, the API streams partial message deltas as they become available, similar to ChatGPT's streaming mode.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateChatCompletionResponse]
            A chat completion
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/llms/chat/completions",
            base_url=self._client_wrapper.get_environment().default,
            method="POST",
            json={
                "model": model,
                "messages": convert_and_respect_annotation_metadata(
                    object_=messages, annotation=typing.Sequence[ChatCompletionRequestMessage], direction="write"
                ),
                "stream": stream,
            },
            headers={
                "content-type": "application/json",
                "Request-Timeout": str(request_timeout) if request_timeout is not None else None,
                "Request-Timeout-Millis": str(request_timeout_millis) if request_timeout_millis is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateChatCompletionResponse,
                    parse_obj_as(
                        type_=CreateChatCompletionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        BadRequestErrorBody,
                        parse_obj_as(
                            type_=BadRequestErrorBody,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .table_extractor import TableExtractor


class ListTableExtractorsResponse(UniversalBaseModel):
    """
    Response containing a list of available table extractors for document processing
    """

    table_extractors: typing.Optional[typing.List[TableExtractor]] = pydantic.Field(default=None)
    """
    An array of table extractors.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    ApiKey,
    ApiKeyRole,
    ApiOperationPolicy,
    ApiPolicy,
    ApiRole,
    AppClient,
    BadRequestErrorBody,
    BearerAuth,
    Cell,
    ChainReranker,
    Chat,
    ChatCompletionRequestMessage,
    ChatCompletionResponseChoice,
    ChatCompletionResponseMessage,
    ChatCompletionStreamResponseChoice,
    ChatCompletionStreamResponseDelta,
    ChatFullResponse,
    ChatInfoResponse,
    ChatParameters,
    ChatStreamedResponse,
    ChunkingStrategy,
    CitationParameters,
    CitationParametersStyle,
    ComponentsSchemasCreateClientCredentialsRequest,
    ComponentsSchemasCreateOpenAiEncoderRequest,
    ComponentsSchemasCreateOpenAillmRequest,
    ComputeCorpusSizeResponse,
    ContextConfiguration,
    CoreDocument,
    CoreDocumentPart,
    Corpus,
    CorpusCustomDimension,
    CorpusKey,
    CorpusLimits,
    CreateChatCompletionResponse,
    CreateChatCompletionStreamResponse,
    CreateClientCredentialsRequest,
    CreateDocumentRequest,
    CreateOpenAiEncoderRequest,
    CreateOpenAillmRequest,
    CustomDimensions,
    CustomerSpecificReranker,
    Data,
    Document,
    DocumentPart,
    DocumentStorageUsage,
    Encoder,
    Error,
    EvaluateFactualConsistencyResponse,
    ExtractionUsage,
    FactualConsistencyScore,
    FactualConsistencyScoreSpan,
    FilterAttribute,
    FilterAttributeLevel,
    FilterAttributeType,
    FilterExtraction,
    GenerationInfo,
    GenerationParameters,
    GenerationPreset,
    GenerationSpan,
    Header,
    HeaderAuth,
    IndividualSearchResult,
    Job,
    JobState,
    JobType,
    KeyedSearchCorpus,
    Language,
    ListApiKeysResponse,
    ListAppClientsResponse,
    ListChatTurnsResponse,
    ListChatsResponse,
    ListCorporaResponse,
    ListDocumentsResponse,
    ListEncodersResponse,
    ListGenerationPresetsResponse,
    ListJobsResponse,
    ListLlMsResponse,
    ListMetadata,
    ListQueryHistoriesResponse,
    ListQueryHistoriesResponseMetadata,
    ListRerankersResponse,
    ListTableExtractorsResponse,
    ListUsersResponse,
    Llm,
    MaxCharsChunkingStrategy,
    MmrReranker,
    ModelParameters,
    NoneReranker,
    NotFoundErrorBody,
    Prompt,
    QueryFullResponse,
    QueryHistory,
    QueryHistorySpan,
    QueryHistorySummary,
    QueryStreamedResponse,
    QueryWarning,
    RemoteAuth,
    RephraseSpan,
    ReplaceFilterAttributesResponse,
    RerankSpan,
    RerankedSearchResult,
    Reranker,
    RewrittenQuery,
    RewrittenQuerySpan,
    RewrittenQueryWarning,
    Row,
    SearchCorporaParameters,
    SearchCorpus,
    SearchParameters,
    SearchReranker,
    SearchSemantics,
    SearchSpan,
    SentenceChunkingStrategy,
    StreamError,
    StreamGenerationChunk,
    StreamGenerationEnd,
    StreamResponseEnd,
    StreamSearchResponse,
    StructuredDocument,
    StructuredDocumentSection,
    SummarizeDocumentResponse,
    SummarizeDocumentStreamedResponse,
    Table,
    TableExtractionConfig,
    TableExtractor,
    TableExtractorSpec,
    TableGenerationSpec,
    Turn,
    UpdateDocumentRequest,
    User,
    UserFunctionReranker,
)
from .errors import (
    BadRequestError,
    ConflictError,
    ForbiddenError,
    NotFoundError,
    TooManyRequestsError,
    UnprocessableEntityError,
    UnsupportedMediaTypeError,
)
from . import (
    api_keys,
    app_clients,
    auth,
    chats,
    corpora,
    documents,
    encoders,
    factual_consistency,
    generation_presets,
    jobs,
    llm,
    llms,
    queries,
    query_history,
    rerankers,
    table_extractors,
    upload,
    users,
)
from .auth import GetTokenResponse
from .client import AsyncVectara, Vectara
from .corpora import SearchCorpusParameters
from .environment import VectaraEnvironment
from .factual_consistency import EvaluateFactualConsistencyRequestModelParameters
from .users import UsersCreateResponse, UsersResetPasswordResponse
from .version import __version__

__all__ = [
    "ApiKey",
    "ApiKeyRole",
    "ApiOperationPolicy",
    "ApiPolicy",
    "ApiRole",
    "AppClient",
    "AsyncVectara",
    "BadRequestError",
    "BadRequestErrorBody",
    "BearerAuth",
    "Cell",
    "ChainReranker",
    "Chat",
    "ChatCompletionRequestMessage",
    "ChatCompletionResponseChoice",
    "ChatCompletionResponseMessage",
    "ChatCompletionStreamResponseChoice",
    "ChatCompletionStreamResponseDelta",
    "ChatFullResponse",
    "ChatInfoResponse",
    "ChatParameters",
    "ChatStreamedResponse",
    "ChunkingStrategy",
    "CitationParameters",
    "CitationParametersStyle",
    "ComponentsSchemasCreateClientCredentialsRequest",
    "ComponentsSchemasCreateOpenAiEncoderRequest",
    "ComponentsSchemasCreateOpenAillmRequest",
    "ComputeCorpusSizeResponse",
    "ConflictError",
    "ContextConfiguration",
    "CoreDocument",
    "CoreDocumentPart",
    "Corpus",
    "CorpusCustomDimension",
    "CorpusKey",
    "CorpusLimits",
    "CreateChatCompletionResponse",
    "CreateChatCompletionStreamResponse",
    "CreateClientCredentialsRequest",
    "CreateDocumentRequest",
    "CreateOpenAiEncoderRequest",
    "CreateOpenAillmRequest",
    "CustomDimensions",
    "CustomerSpecificReranker",
    "Data",
    "Document",
    "DocumentPart",
    "DocumentStorageUsage",
    "Encoder",
    "Error",
    "EvaluateFactualConsistencyRequestModelParameters",
    "EvaluateFactualConsistencyResponse",
    "ExtractionUsage",
    "FactualConsistencyScore",
    "FactualConsistencyScoreSpan",
    "FilterAttribute",
    "FilterAttributeLevel",
    "FilterAttributeType",
    "FilterExtraction",
    "ForbiddenError",
    "GenerationInfo",
    "GenerationParameters",
    "GenerationPreset",
    "GenerationSpan",
    "GetTokenResponse",
    "Header",
    "HeaderAuth",
    "IndividualSearchResult",
    "Job",
    "JobState",
    "JobType",
    "KeyedSearchCorpus",
    "Language",
    "ListApiKeysResponse",
    "ListAppClientsResponse",
    "ListChatTurnsResponse",
    "ListChatsResponse",
    "ListCorporaResponse",
    "ListDocumentsResponse",
    "ListEncodersResponse",
    "ListGenerationPresetsResponse",
    "ListJobsResponse",
    "ListLlMsResponse",
    "ListMetadata",
    "ListQueryHistoriesResponse",
    "ListQueryHistoriesResponseMetadata",
    "ListRerankersResponse",
    "ListTableExtractorsResponse",
    "ListUsersResponse",
    "Llm",
    "MaxCharsChunkingStrategy",
    "MmrReranker",
    "ModelParameters",
    "NoneReranker",
    "NotFoundError",
    "NotFoundErrorBody",
    "Prompt",
    "QueryFullResponse",
    "QueryHistory",
    "QueryHistorySpan",
    "QueryHistorySummary",
    "QueryStreamedResponse",
    "QueryWarning",
    "RemoteAuth",
    "RephraseSpan",
    "ReplaceFilterAttributesResponse",
    "RerankSpan",
    "RerankedSearchResult",
    "Reranker",
    "RewrittenQuery",
    "RewrittenQuerySpan",
    "RewrittenQueryWarning",
    "Row",
    "SearchCorporaParameters",
    "SearchCorpus",
    "SearchCorpusParameters",
    "SearchParameters",
    "SearchReranker",
    "SearchSemantics",
    "SearchSpan",
    "SentenceChunkingStrategy",
    "StreamError",
    "StreamGenerationChunk",
    "StreamGenerationEnd",
    "StreamResponseEnd",
    "StreamSearchResponse",
    "StructuredDocument",
    "StructuredDocumentSection",
    "SummarizeDocumentResponse",
    "SummarizeDocumentStreamedResponse",
    "Table",
    "TableExtractionConfig",
    "TableExtractor",
    "TableExtractorSpec",
    "TableGenerationSpec",
    "TooManyRequestsError",
    "Turn",
    "UnprocessableEntityError",
    "UnsupportedMediaTypeError",
    "UpdateDocumentRequest",
    "User",
    "UserFunctionReranker",
    "UsersCreateResponse",
    "UsersResetPasswordResponse",
    "Vectara",
    "VectaraEnvironment",
    "__version__",
    "api_keys",
    "app_clients",
    "auth",
    "chats",
    "corpora",
    "documents",
    "encoders",
    "factual_consistency",
    "generation_presets",
    "jobs",
    "llm",
    "llms",
    "queries",
    "query_history",
    "rerankers",
    "table_extractors",
    "upload",
    "users",
]



# This file was auto-generated by Fern from our API Definition.

import typing

QueryWarning = typing.Union[
    typing.Literal["exceeded_max_input_length_fcs", "intelligent_query_rewriting_failed"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .chat_completion_stream_response_choice import ChatCompletionStreamResponseChoice


class CreateChatCompletionStreamResponse(UniversalBaseModel):
    """
    The response object for streaming chat completions in chunks.
    """

    object: typing.Literal["chat.completion.chunk"] = pydantic.Field(default="chat.completion.chunk")
    """
    The object type, which is always 'chat.completion.chunk' for streaming responses.
    """

    choices: typing.List[ChatCompletionStreamResponseChoice] = pydantic.Field()
    """
    A list of chat completion chunks.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.corpus_key import CorpusKey
from ..types.job import Job
from ..types.job_state import JobState
from .raw_client import AsyncRawJobsClient, RawJobsClient


class JobsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawJobsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawJobsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawJobsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        corpus_key: typing.Optional[typing.Union[CorpusKey, typing.Sequence[CorpusKey]]] = None,
        after: typing.Optional[dt.datetime] = None,
        state: typing.Optional[typing.Union[JobState, typing.Sequence[JobState]]] = None,
        limit: typing.Optional[int] = None,
        page_key: typing.Optional[str] = None,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Job]:
        """
        List jobs for the account. Jobs are background processes like replacing the filterable metadata attributes.

        Parameters
        ----------
        corpus_key : typing.Optional[typing.Union[CorpusKey, typing.Sequence[CorpusKey]]]
            The unique key identifying the corpus with the job.

        after : typing.Optional[dt.datetime]
            Filter by jobs created after a particular date-time.

        state : typing.Optional[typing.Union[JobState, typing.Sequence[JobState]]]
            Filter by jobs in particular states.

        limit : typing.Optional[int]
            The maximum number of jobs to return at one time.

        page_key : typing.Optional[str]
            Used to retrieve the next page of jobs after the limit has been reached.

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Job]
            List of jobs.

        Examples
        --------
        from vectara import Vectara
        client = Vectara(api_key="YOUR_API_KEY", client_id="YOUR_CLIENT_ID", client_secret="YOUR_CLIENT_SECRET", )
        response = client.jobs.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        response = self._raw_client.list(
            corpus_key=corpus_key,
            after=after,
            state=state,
            limit=limit,
            page_key=page_key,
            request_timeout=request_timeout,
            request_timeout_millis=request_timeout_millis,
            request_options=request_options,
        )
        return response.data

    def get(
        self,
        job_id: str,
        *,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Job:
        """
        Get a job by a specific ID. Jobs are background processes like replacing the filterable metadata attributes.

        Parameters
        ----------
        job_id : str
            The ID of the job to get.

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Job
            A job.

        Examples
        --------
        from vectara import Vectara
        client = Vectara(api_key="YOUR_API_KEY", client_id="YOUR_CLIENT_ID", client_secret="YOUR_CLIENT_SECRET", )
        client.jobs.get(job_id='job_id', )
        """
        response = self._raw_client.get(
            job_id,
            request_timeout=request_timeout,
            request_timeout_millis=request_timeout_millis,
            request_options=request_options,
        )
        return response.data


class AsyncJobsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawJobsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawJobsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawJobsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        corpus_key: typing.Optional[typing.Union[CorpusKey, typing.Sequence[CorpusKey]]] = None,
        after: typing.Optional[dt.datetime] = None,
        state: typing.Optional[typing.Union[JobState, typing.Sequence[JobState]]] = None,
        limit: typing.Optional[int] = None,
        page_key: typing.Optional[str] = None,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Job]:
        """
        List jobs for the account. Jobs are background processes like replacing the filterable metadata attributes.

        Parameters
        ----------
        corpus_key : typing.Optional[typing.Union[CorpusKey, typing.Sequence[CorpusKey]]]
            The unique key identifying the corpus with the job.

        after : typing.Optional[dt.datetime]
            Filter by jobs created after a particular date-time.

        state : typing.Optional[typing.Union[JobState, typing.Sequence[JobState]]]
            Filter by jobs in particular states.

        limit : typing.Optional[int]
            The maximum number of jobs to return at one time.

        page_key : typing.Optional[str]
            Used to retrieve the next page of jobs after the limit has been reached.

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Job]
            List of jobs.

        Examples
        --------
        from vectara import AsyncVectara
        import asyncio
        client = AsyncVectara(api_key="YOUR_API_KEY", client_id="YOUR_CLIENT_ID", client_secret="YOUR_CLIENT_SECRET", )
        async def main() -> None:
            response = await client.jobs.list()
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page
        asyncio.run(main())
        """
        response = await self._raw_client.list(
            corpus_key=corpus_key,
            after=after,
            state=state,
            limit=limit,
            page_key=page_key,
            request_timeout=request_timeout,
            request_timeout_millis=request_timeout_millis,
            request_options=request_options,
        )
        return response.data

    async def get(
        self,
        job_id: str,
        *,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Job:
        """
        Get a job by a specific ID. Jobs are background processes like replacing the filterable metadata attributes.

        Parameters
        ----------
        job_id : str
            The ID of the job to get.

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Job
            A job.

        Examples
        --------
        from vectara import AsyncVectara
        import asyncio
        client = AsyncVectara(api_key="YOUR_API_KEY", client_id="YOUR_CLIENT_ID", client_secret="YOUR_CLIENT_SECRET", )
        async def main() -> None:
            await client.jobs.get(job_id='job_id', )
        asyncio.run(main())
        """
        response = await self._raw_client.get(
            job_id,
            request_timeout=request_timeout,
            request_timeout_millis=request_timeout_millis,
            request_options=request_options,
        )
        return response.data



# This file was auto-generated by Fern from our API Definition.

import contextlib
import typing
from json.decoder import JSONDecodeError

import httpx_sse
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..types.bad_request_error_body import BadRequestErrorBody
from ..types.error import Error
from ..types.generation_parameters import GenerationParameters
from ..types.not_found_error_body import NotFoundErrorBody
from ..types.query_full_response import QueryFullResponse
from ..types.query_streamed_response import QueryStreamedResponse
from ..types.search_corpora_parameters import SearchCorporaParameters

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawQueriesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    @contextlib.contextmanager
    def query_stream(
        self,
        *,
        query: str,
        search: SearchCorporaParameters,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        generation: typing.Optional[GenerationParameters] = OMIT,
        save_history: typing.Optional[bool] = OMIT,
        intelligent_query_rewriting: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[HttpResponse[typing.Iterator[QueryStreamedResponse]]]:
        """
        Perform a multipurpose query across to retrieve relevant information from one or more corpora and generate a response using Retrieval Augmented Generation (RAG).

        * Specify the unique `corpus_key` identifying the corpus to query. The `corpus_key` is [created in the Vectara Console UI](https://docs.vectara.com/docs/console-ui/creating-a-corpus) or the [Create Corpus API definition](https://docs.vectara.com/docs/api-reference/admin-apis/create-corpus). When creating a new corpus, you have the option to assign a custom `corpus_key` following your preferred naming convention. This key serves as a unique identifier for the corpus, allowing it to be referenced in search requests. For more information, see [Corpus Key Definition](https://docs.vectara.com/docs/api-reference/search-apis/search#corpus-key-definition).
        * Customize your search by specifying the query text (`query`), pagination details (`offset` and `limit`), and metadata filters (`metadata_filter`) to tailor your search results. [Learn more](https://docs.vectara.com/docs/api-reference/search-apis/search#query-definition)
        * Leverage advanced search capabilities like reranking (`reranker`) and opt-in Retrieval Augmented Generation (RAG) (`generation`) for enhanced query performance. Generation is opt in by setting the `generation` property. By excluding the property or by setting it to null, the response
        will not include generation. [Learn more](https://docs.vectara.com/docs/learn/grounded-generation/configure-query-summarization)
        * Specify Vectara's RAG-focused LLM (Mockingbird) for the `generation_preset_name`. [Learn more](https://docs.vectara.com/docs/learn/mockingbird-llm)
        * Use advanced summarization options that utilize detailed summarization parameters such as `max_response_characters`, `temperature`, and `frequency_penalty` for generating precise and relevant summaries. [Learn more](https://docs.vectara.com/docs/api-reference/search-apis/search#advanced-summarization-customization-options)
        * Customize citation formats in summaries using the `citations` object to include numeric, HTML, or Markdown links. [Learn more](https://docs.vectara.com/docs/api-reference/search-apis/search#citation-format-in-summary)

        For more detailed information, see this [Query API guide](https://docs.vectara.com/docs/api-reference/search-apis/search).

        Parameters
        ----------
        query : str
            The search query string, which is the question the user is asking.

        search : SearchCorporaParameters

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        generation : typing.Optional[GenerationParameters]

        save_history : typing.Optional[bool]
            Indicates whether to save the query to query history.

        intelligent_query_rewriting : typing.Optional[bool]
            Indicates whether to enable intelligent query rewriting. When enabled, the platform will attempt to
            extract metadata filter and rewrite the query to improve search results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.Iterator[HttpResponse[typing.Iterator[QueryStreamedResponse]]]

        """
        with self._client_wrapper.httpx_client.stream(
            "v2/query",
            base_url=self._client_wrapper.get_environment().default,
            method="POST",
            json={
                "query": query,
                "search": convert_and_respect_annotation_metadata(
                    object_=search, annotation=SearchCorporaParameters, direction="write"
                ),
                "generation": convert_and_respect_annotation_metadata(
                    object_=generation, annotation=GenerationParameters, direction="write"
                ),
                "save_history": save_history,
                "intelligent_query_rewriting": intelligent_query_rewriting,
                "stream_response": True,
            },
            headers={
                "content-type": "application/json",
                "Request-Timeout": str(request_timeout) if request_timeout is not None else None,
                "Request-Timeout-Millis": str(request_timeout_millis) if request_timeout_millis is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            def stream() -> HttpResponse[typing.Iterator[QueryStreamedResponse]]:
                try:
                    if 200 <= _response.status_code < 300:

                        def _iter():
                            _event_source = httpx_sse.EventSource(_response)
                            for _sse in _event_source.iter_sse():
                                if _sse.data == None:
                                    return
                                try:
                                    yield _sse.data()
                                except Exception:
                                    pass
                            return

                        return HttpResponse(response=_response, data=_iter())
                    _response.read()
                    if _response.status_code == 400:
                        raise BadRequestError(
                            typing.cast(
                                BadRequestErrorBody,
                                parse_obj_as(
                                    type_=BadRequestErrorBody,  # type: ignore
                                    object_=_response.json(),
                                ),
                            )
                        )
                    if _response.status_code == 403:
                        raise ForbiddenError(
                            typing.cast(
                                Error,
                                parse_obj_as(
                                    type_=Error,  # type: ignore
                                    object_=_response.json(),
                                ),
                            )
                        )
                    if _response.status_code == 404:
                        raise NotFoundError(
                            typing.cast(
                                NotFoundErrorBody,
                                parse_obj_as(
                                    type_=NotFoundErrorBody,  # type: ignore
                                    object_=_response.json(),
                                ),
                            )
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(status_code=_response.status_code, body=_response.text)
                raise ApiError(status_code=_response.status_code, body=_response_json)

            yield stream()

    def query(
        self,
        *,
        query: str,
        search: SearchCorporaParameters,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        generation: typing.Optional[GenerationParameters] = OMIT,
        save_history: typing.Optional[bool] = OMIT,
        intelligent_query_rewriting: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[QueryFullResponse]:
        """
        Perform a multipurpose query across to retrieve relevant information from one or more corpora and generate a response using Retrieval Augmented Generation (RAG).

        * Specify the unique `corpus_key` identifying the corpus to query. The `corpus_key` is [created in the Vectara Console UI](https://docs.vectara.com/docs/console-ui/creating-a-corpus) or the [Create Corpus API definition](https://docs.vectara.com/docs/api-reference/admin-apis/create-corpus). When creating a new corpus, you have the option to assign a custom `corpus_key` following your preferred naming convention. This key serves as a unique identifier for the corpus, allowing it to be referenced in search requests. For more information, see [Corpus Key Definition](https://docs.vectara.com/docs/api-reference/search-apis/search#corpus-key-definition).
        * Customize your search by specifying the query text (`query`), pagination details (`offset` and `limit`), and metadata filters (`metadata_filter`) to tailor your search results. [Learn more](https://docs.vectara.com/docs/api-reference/search-apis/search#query-definition)
        * Leverage advanced search capabilities like reranking (`reranker`) and opt-in Retrieval Augmented Generation (RAG) (`generation`) for enhanced query performance. Generation is opt in by setting the `generation` property. By excluding the property or by setting it to null, the response
        will not include generation. [Learn more](https://docs.vectara.com/docs/learn/grounded-generation/configure-query-summarization)
        * Specify Vectara's RAG-focused LLM (Mockingbird) for the `generation_preset_name`. [Learn more](https://docs.vectara.com/docs/learn/mockingbird-llm)
        * Use advanced summarization options that utilize detailed summarization parameters such as `max_response_characters`, `temperature`, and `frequency_penalty` for generating precise and relevant summaries. [Learn more](https://docs.vectara.com/docs/api-reference/search-apis/search#advanced-summarization-customization-options)
        * Customize citation formats in summaries using the `citations` object to include numeric, HTML, or Markdown links. [Learn more](https://docs.vectara.com/docs/api-reference/search-apis/search#citation-format-in-summary)

        For more detailed information, see this [Query API guide](https://docs.vectara.com/docs/api-reference/search-apis/search).

        Parameters
        ----------
        query : str
            The search query string, which is the question the user is asking.

        search : SearchCorporaParameters

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        generation : typing.Optional[GenerationParameters]

        save_history : typing.Optional[bool]
            Indicates whether to save the query to query history.

        intelligent_query_rewriting : typing.Optional[bool]
            Indicates whether to enable intelligent query rewriting. When enabled, the platform will attempt to
            extract metadata filter and rewrite the query to improve search results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[QueryFullResponse]

        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/query",
            base_url=self._client_wrapper.get_environment().default,
            method="POST",
            json={
                "query": query,
                "search": convert_and_respect_annotation_metadata(
                    object_=search, annotation=SearchCorporaParameters, direction="write"
                ),
                "generation": convert_and_respect_annotation_metadata(
                    object_=generation, annotation=GenerationParameters, direction="write"
                ),
                "save_history": save_history,
                "intelligent_query_rewriting": intelligent_query_rewriting,
                "stream_response": False,
            },
            headers={
                "content-type": "application/json",
                "Request-Timeout": str(request_timeout) if request_timeout is not None else None,
                "Request-Timeout-Millis": str(request_timeout_millis) if request_timeout_millis is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    QueryFullResponse,
                    parse_obj_as(
                        type_=QueryFullResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        BadRequestErrorBody,
                        parse_obj_as(
                            type_=BadRequestErrorBody,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        NotFoundErrorBody,
                        parse_obj_as(
                            type_=NotFoundErrorBody,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawQueriesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    @contextlib.asynccontextmanager
    async def query_stream(
        self,
        *,
        query: str,
        search: SearchCorporaParameters,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        generation: typing.Optional[GenerationParameters] = OMIT,
        save_history: typing.Optional[bool] = OMIT,
        intelligent_query_rewriting: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[QueryStreamedResponse]]]:
        """
        Perform a multipurpose query across to retrieve relevant information from one or more corpora and generate a response using Retrieval Augmented Generation (RAG).

        * Specify the unique `corpus_key` identifying the corpus to query. The `corpus_key` is [created in the Vectara Console UI](https://docs.vectara.com/docs/console-ui/creating-a-corpus) or the [Create Corpus API definition](https://docs.vectara.com/docs/api-reference/admin-apis/create-corpus). When creating a new corpus, you have the option to assign a custom `corpus_key` following your preferred naming convention. This key serves as a unique identifier for the corpus, allowing it to be referenced in search requests. For more information, see [Corpus Key Definition](https://docs.vectara.com/docs/api-reference/search-apis/search#corpus-key-definition).
        * Customize your search by specifying the query text (`query`), pagination details (`offset` and `limit`), and metadata filters (`metadata_filter`) to tailor your search results. [Learn more](https://docs.vectara.com/docs/api-reference/search-apis/search#query-definition)
        * Leverage advanced search capabilities like reranking (`reranker`) and opt-in Retrieval Augmented Generation (RAG) (`generation`) for enhanced query performance. Generation is opt in by setting the `generation` property. By excluding the property or by setting it to null, the response
        will not include generation. [Learn more](https://docs.vectara.com/docs/learn/grounded-generation/configure-query-summarization)
        * Specify Vectara's RAG-focused LLM (Mockingbird) for the `generation_preset_name`. [Learn more](https://docs.vectara.com/docs/learn/mockingbird-llm)
        * Use advanced summarization options that utilize detailed summarization parameters such as `max_response_characters`, `temperature`, and `frequency_penalty` for generating precise and relevant summaries. [Learn more](https://docs.vectara.com/docs/api-reference/search-apis/search#advanced-summarization-customization-options)
        * Customize citation formats in summaries using the `citations` object to include numeric, HTML, or Markdown links. [Learn more](https://docs.vectara.com/docs/api-reference/search-apis/search#citation-format-in-summary)

        For more detailed information, see this [Query API guide](https://docs.vectara.com/docs/api-reference/search-apis/search).

        Parameters
        ----------
        query : str
            The search query string, which is the question the user is asking.

        search : SearchCorporaParameters

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        generation : typing.Optional[GenerationParameters]

        save_history : typing.Optional[bool]
            Indicates whether to save the query to query history.

        intelligent_query_rewriting : typing.Optional[bool]
            Indicates whether to enable intelligent query rewriting. When enabled, the platform will attempt to
            extract metadata filter and rewrite the query to improve search results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[QueryStreamedResponse]]]

        """
        async with self._client_wrapper.httpx_client.stream(
            "v2/query",
            base_url=self._client_wrapper.get_environment().default,
            method="POST",
            json={
                "query": query,
                "search": convert_and_respect_annotation_metadata(
                    object_=search, annotation=SearchCorporaParameters, direction="write"
                ),
                "generation": convert_and_respect_annotation_metadata(
                    object_=generation, annotation=GenerationParameters, direction="write"
                ),
                "save_history": save_history,
                "intelligent_query_rewriting": intelligent_query_rewriting,
                "stream_response": True,
            },
            headers={
                "content-type": "application/json",
                "Request-Timeout": str(request_timeout) if request_timeout is not None else None,
                "Request-Timeout-Millis": str(request_timeout_millis) if request_timeout_millis is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            async def stream() -> AsyncHttpResponse[typing.AsyncIterator[QueryStreamedResponse]]:
                try:
                    if 200 <= _response.status_code < 300:

                        async def _iter():
                            _event_source = httpx_sse.EventSource(_response)
                            async for _sse in _event_source.aiter_sse():
                                if _sse.data == None:
                                    return
                                try:
                                    yield _sse.data()
                                except Exception:
                                    pass
                            return

                        return AsyncHttpResponse(response=_response, data=_iter())
                    await _response.aread()
                    if _response.status_code == 400:
                        raise BadRequestError(
                            typing.cast(
                                BadRequestErrorBody,
                                parse_obj_as(
                                    type_=BadRequestErrorBody,  # type: ignore
                                    object_=_response.json(),
                                ),
                            )
                        )
                    if _response.status_code == 403:
                        raise ForbiddenError(
                            typing.cast(
                                Error,
                                parse_obj_as(
                                    type_=Error,  # type: ignore
                                    object_=_response.json(),
                                ),
                            )
                        )
                    if _response.status_code == 404:
                        raise NotFoundError(
                            typing.cast(
                                NotFoundErrorBody,
                                parse_obj_as(
                                    type_=NotFoundErrorBody,  # type: ignore
                                    object_=_response.json(),
                                ),
                            )
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(status_code=_response.status_code, body=_response.text)
                raise ApiError(status_code=_response.status_code, body=_response_json)

            yield await stream()

    async def query(
        self,
        *,
        query: str,
        search: SearchCorporaParameters,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        generation: typing.Optional[GenerationParameters] = OMIT,
        save_history: typing.Optional[bool] = OMIT,
        intelligent_query_rewriting: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[QueryFullResponse]:
        """
        Perform a multipurpose query across to retrieve relevant information from one or more corpora and generate a response using Retrieval Augmented Generation (RAG).

        * Specify the unique `corpus_key` identifying the corpus to query. The `corpus_key` is [created in the Vectara Console UI](https://docs.vectara.com/docs/console-ui/creating-a-corpus) or the [Create Corpus API definition](https://docs.vectara.com/docs/api-reference/admin-apis/create-corpus). When creating a new corpus, you have the option to assign a custom `corpus_key` following your preferred naming convention. This key serves as a unique identifier for the corpus, allowing it to be referenced in search requests. For more information, see [Corpus Key Definition](https://docs.vectara.com/docs/api-reference/search-apis/search#corpus-key-definition).
        * Customize your search by specifying the query text (`query`), pagination details (`offset` and `limit`), and metadata filters (`metadata_filter`) to tailor your search results. [Learn more](https://docs.vectara.com/docs/api-reference/search-apis/search#query-definition)
        * Leverage advanced search capabilities like reranking (`reranker`) and opt-in Retrieval Augmented Generation (RAG) (`generation`) for enhanced query performance. Generation is opt in by setting the `generation` property. By excluding the property or by setting it to null, the response
        will not include generation. [Learn more](https://docs.vectara.com/docs/learn/grounded-generation/configure-query-summarization)
        * Specify Vectara's RAG-focused LLM (Mockingbird) for the `generation_preset_name`. [Learn more](https://docs.vectara.com/docs/learn/mockingbird-llm)
        * Use advanced summarization options that utilize detailed summarization parameters such as `max_response_characters`, `temperature`, and `frequency_penalty` for generating precise and relevant summaries. [Learn more](https://docs.vectara.com/docs/api-reference/search-apis/search#advanced-summarization-customization-options)
        * Customize citation formats in summaries using the `citations` object to include numeric, HTML, or Markdown links. [Learn more](https://docs.vectara.com/docs/api-reference/search-apis/search#citation-format-in-summary)

        For more detailed information, see this [Query API guide](https://docs.vectara.com/docs/api-reference/search-apis/search).

        Parameters
        ----------
        query : str
            The search query string, which is the question the user is asking.

        search : SearchCorporaParameters

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        generation : typing.Optional[GenerationParameters]

        save_history : typing.Optional[bool]
            Indicates whether to save the query to query history.

        intelligent_query_rewriting : typing.Optional[bool]
            Indicates whether to enable intelligent query rewriting. When enabled, the platform will attempt to
            extract metadata filter and rewrite the query to improve search results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[QueryFullResponse]

        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/query",
            base_url=self._client_wrapper.get_environment().default,
            method="POST",
            json={
                "query": query,
                "search": convert_and_respect_annotation_metadata(
                    object_=search, annotation=SearchCorporaParameters, direction="write"
                ),
                "generation": convert_and_respect_annotation_metadata(
                    object_=generation, annotation=GenerationParameters, direction="write"
                ),
                "save_history": save_history,
                "intelligent_query_rewriting": intelligent_query_rewriting,
                "stream_response": False,
            },
            headers={
                "content-type": "application/json",
                "Request-Timeout": str(request_timeout) if request_timeout is not None else None,
                "Request-Timeout-Millis": str(request_timeout_millis) if request_timeout_millis is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    QueryFullResponse,
                    parse_obj_as(
                        type_=QueryFullResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        BadRequestErrorBody,
                        parse_obj_as(
                            type_=BadRequestErrorBody,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        NotFoundErrorBody,
                        parse_obj_as(
                            type_=NotFoundErrorBody,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .corpus_key import CorpusKey
from .search_corpus import SearchCorpus


class KeyedSearchCorpus(SearchCorpus):
    """
    A corpus with its identifying key for use in search operations within a customer account.
    """

    corpus_key: typing.Optional[CorpusKey] = None
    query: typing.Optional[str] = pydantic.Field(default=None)
    """
    Query for a particular corpus that will override the overall query if present.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.corpus_key import CorpusKey
from ..types.create_document_request import CreateDocumentRequest
from ..types.document import Document
from ..types.summarize_document_response import SummarizeDocumentResponse
from .raw_client import AsyncRawDocumentsClient, RawDocumentsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DocumentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDocumentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDocumentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDocumentsClient
        """
        return self._raw_client

    def list(
        self,
        corpus_key: CorpusKey,
        *,
        limit: typing.Optional[int] = None,
        metadata_filter: typing.Optional[str] = None,
        page_key: typing.Optional[str] = None,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Document]:
        """
        Retrieve a list of documents stored in a specific corpus. This endpoint
        provides an overview of document metadata without returning the full content of
        each document.

        Parameters
        ----------
        corpus_key : CorpusKey
            The unique key identifying the queried corpus.

        limit : typing.Optional[int]
            The maximum number of documents to return at one time.

        metadata_filter : typing.Optional[str]
            Filter documents by metadata. Uses the same expression as a query metadata filter, but only
            allows filtering on document metadata.

        page_key : typing.Optional[str]
            Used to retrieve the next page of documents after the limit has been reached.

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Document]
            List of documents.

        Examples
        --------
        from vectara import Vectara
        client = Vectara(api_key="YOUR_API_KEY", client_id="YOUR_CLIENT_ID", client_secret="YOUR_CLIENT_SECRET", )
        response = client.documents.list(corpus_key='my-corpus', )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        response = self._raw_client.list(
            corpus_key,
            limit=limit,
            metadata_filter=metadata_filter,
            page_key=page_key,
            request_timeout=request_timeout,
            request_timeout_millis=request_timeout_millis,
            request_options=request_options,
        )
        return response.data

    def create(
        self,
        corpus_key: CorpusKey,
        *,
        request: CreateDocumentRequest,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Document:
        """
        Add a document to a corpus. This endpoint supports two document formats, structured and core.

        * **Structured** documents have a more conventional structure that provide document sections
        and parts in a format created by Vectara's proprietary strategy automatically. You provide
        a logical document structure, and Vectara handles the partitioning.
        * **Core** documents differ in that they follow an advanced, granular structure that
        explicitly defines each document part in an array. Each part becomes a distinct,
        searchable item in query results. You have precise control over the document structure
        and content.

        For more details, see [Indexing](https://docs.vectara.com/docs/learn/select-ideal-indexing-api).

        Parameters
        ----------
        corpus_key : CorpusKey
            The unique key identifying the queried corpus.

        request : CreateDocumentRequest

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Document
            Document added to the corpus.

        Examples
        --------
        from vectara import Vectara
        from vectara import StructuredDocument
        from vectara import StructuredDocumentSection
        client = Vectara(api_key="YOUR_API_KEY", client_id="YOUR_CLIENT_ID", client_secret="YOUR_CLIENT_SECRET", )
        client.documents.create(corpus_key='my-corpus-key', request=StructuredDocument(id='my-doc-id', sections=[StructuredDocumentSection(id=1, title='A nice title.', text="I'm a nice document section.", metadata={'section': '1.1'
        }, ), StructuredDocumentSection(id=2, title='Another nice title.', text="I'm another document section on something else.", metadata={'section': '1.2'
        }, )], metadata={'url': 'https://example.com'
        }, ), )
        """
        response = self._raw_client.create(
            corpus_key,
            request=request,
            request_timeout=request_timeout,
            request_timeout_millis=request_timeout_millis,
            request_options=request_options,
        )
        return response.data

    def get(
        self,
        corpus_key: CorpusKey,
        document_id: str,
        *,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Document:
        """
        Retrieve the content and metadata of a specific document, identified by its
        unique `document_id` from a specific corpus.

        Parameters
        ----------
        corpus_key : CorpusKey
            The unique key identifying the corpus containing the document to retrieve.

        document_id : str
            The document ID of the document to retrieve.
            This `document_id` must be percent encoded.

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Document
            Successfully retrieved the document.

        Examples
        --------
        from vectara import Vectara
        client = Vectara(api_key="YOUR_API_KEY", client_id="YOUR_CLIENT_ID", client_secret="YOUR_CLIENT_SECRET", )
        client.documents.get(corpus_key='my-corpus', document_id='document_id', )
        """
        response = self._raw_client.get(
            corpus_key,
            document_id,
            request_timeout=request_timeout,
            request_timeout_millis=request_timeout_millis,
            request_options=request_options,
        )
        return response.data

    def delete(
        self,
        corpus_key: CorpusKey,
        document_id: str,
        *,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Permanently delete a document identified by its unique `document_id` from a specific
        corpus. This operation cannot be undone, so use it with caution.

        Parameters
        ----------
        corpus_key : CorpusKey
            The unique key identifying the corpus with the document to delete.

        document_id : str
            The document ID of the document to delete.
            This `document_id` must be percent encoded.

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from vectara import Vectara
        client = Vectara(api_key="YOUR_API_KEY", client_id="YOUR_CLIENT_ID", client_secret="YOUR_CLIENT_SECRET", )
        client.documents.delete(corpus_key='my-corpus', document_id='document_id', )
        """
        response = self._raw_client.delete(
            corpus_key,
            document_id,
            request_timeout=request_timeout,
            request_timeout_millis=request_timeout_millis,
            request_options=request_options,
        )
        return response.data

    def update(
        self,
        corpus_key: CorpusKey,
        document_id: str,
        *,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Document:
        """
        Updates document identified by its unique `document_id` from a specific
        corpus. The request body metadata is merged with the existing metadata,
        adding or modifying only the specified fields.

        Parameters
        ----------
        corpus_key : CorpusKey
            The unique key identifying the corpus with the document to update.

        document_id : str
            The document ID of the document to update.
            This `document_id` must be percent encoded.

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            The metadata for a document as an arbitrary object. Properties of this object
            can be used by document level filter attributes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Document
            Successfully updated the document.

        Examples
        --------
        from vectara import Vectara
        client = Vectara(api_key="YOUR_API_KEY", client_id="YOUR_CLIENT_ID", client_secret="YOUR_CLIENT_SECRET", )
        client.documents.update(corpus_key='my-corpus', document_id='document_id', )
        """
        response = self._raw_client.update(
            corpus_key,
            document_id,
            request_timeout=request_timeout,
            request_timeout_millis=request_timeout_millis,
            metadata=metadata,
            request_options=request_options,
        )
        return response.data

    def update_metadata(
        self,
        corpus_key: CorpusKey,
        document_id: str,
        *,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Document:
        """
        Replaces metadata of a document identified by its unique `document_id`
        from a specific corpus.

        Parameters
        ----------
        corpus_key : CorpusKey
            The unique key identifying the corpus with the document to update.

        document_id : str
            The document ID of the document to update.
            This `document_id` must be percent encoded.

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            The metadata for a document as an arbitrary object. Properties of this object
            can be used by document level filter attributes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Document
            Successfully updated the document.

        Examples
        --------
        from vectara import Vectara
        client = Vectara(api_key="YOUR_API_KEY", client_id="YOUR_CLIENT_ID", client_secret="YOUR_CLIENT_SECRET", )
        client.documents.update_metadata(corpus_key='my-corpus', document_id='document_id', )
        """
        response = self._raw_client.update_metadata(
            corpus_key,
            document_id,
            request_timeout=request_timeout,
            request_timeout_millis=request_timeout_millis,
            metadata=metadata,
            request_options=request_options,
        )
        return response.data

    def summarize(
        self,
        corpus_key: CorpusKey,
        document_id: str,
        *,
        llm_name: str,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        prompt_template: typing.Optional[str] = OMIT,
        model_parameters: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        stream_response: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SummarizeDocumentResponse:
        """
        Summarize a document identified by its unique `document_id` from a specific corpus.

        Parameters
        ----------
        corpus_key : CorpusKey
            The unique key identifying the corpus containing the document to retrieve.

        document_id : str
            The document ID of the document to retrieve.
            This `document_id` must be percent encoded.

        llm_name : str
            The name of the LLM.

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        prompt_template : typing.Optional[str]
            The prompt template to use when generating the summary.
            Vectara manages both system and user roles and prompts for the generative
            LLM out of the box by default. However, users can override the
            `prompt_template` via this variable. The `prompt_template` is in the form of an
            Apache Velocity template. For more details on how to configure the
            `prompt_template`, see the [long-form documentation](https://docs.vectara.com/docs/prompts/vectara-prompt-engine).

        model_parameters : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Optional parameters for the specified model used when generating the summary.

        stream_response : typing.Optional[bool]
            Indicates whether the response should be streamed or not.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SummarizeDocumentResponse
            Document summarization response on success.

        Examples
        --------
        from vectara import Vectara
        client = Vectara(api_key="YOUR_API_KEY", client_id="YOUR_CLIENT_ID", client_secret="YOUR_CLIENT_SECRET", )
        client.documents.summarize(corpus_key='my-corpus', document_id='document_id', llm_name='llm_name', )
        """
        response = self._raw_client.summarize(
            corpus_key,
            document_id,
            llm_name=llm_name,
            request_timeout=request_timeout,
            request_timeout_millis=request_timeout_millis,
            prompt_template=prompt_template,
            model_parameters=model_parameters,
            stream_response=stream_response,
            request_options=request_options,
        )
        return response.data


class AsyncDocumentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDocumentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDocumentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDocumentsClient
        """
        return self._raw_client

    async def list(
        self,
        corpus_key: CorpusKey,
        *,
        limit: typing.Optional[int] = None,
        metadata_filter: typing.Optional[str] = None,
        page_key: typing.Optional[str] = None,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Document]:
        """
        Retrieve a list of documents stored in a specific corpus. This endpoint
        provides an overview of document metadata without returning the full content of
        each document.

        Parameters
        ----------
        corpus_key : CorpusKey
            The unique key identifying the queried corpus.

        limit : typing.Optional[int]
            The maximum number of documents to return at one time.

        metadata_filter : typing.Optional[str]
            Filter documents by metadata. Uses the same expression as a query metadata filter, but only
            allows filtering on document metadata.

        page_key : typing.Optional[str]
            Used to retrieve the next page of documents after the limit has been reached.

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Document]
            List of documents.

        Examples
        --------
        from vectara import AsyncVectara
        import asyncio
        client = AsyncVectara(api_key="YOUR_API_KEY", client_id="YOUR_CLIENT_ID", client_secret="YOUR_CLIENT_SECRET", )
        async def main() -> None:
            response = await client.documents.list(corpus_key='my-corpus', )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page
        asyncio.run(main())
        """
        response = await self._raw_client.list(
            corpus_key,
            limit=limit,
            metadata_filter=metadata_filter,
            page_key=page_key,
            request_timeout=request_timeout,
            request_timeout_millis=request_timeout_millis,
            request_options=request_options,
        )
        return response.data

    async def create(
        self,
        corpus_key: CorpusKey,
        *,
        request: CreateDocumentRequest,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Document:
        """
        Add a document to a corpus. This endpoint supports two document formats, structured and core.

        * **Structured** documents have a more conventional structure that provide document sections
        and parts in a format created by Vectara's proprietary strategy automatically. You provide
        a logical document structure, and Vectara handles the partitioning.
        * **Core** documents differ in that they follow an advanced, granular structure that
        explicitly defines each document part in an array. Each part becomes a distinct,
        searchable item in query results. You have precise control over the document structure
        and content.

        For more details, see [Indexing](https://docs.vectara.com/docs/learn/select-ideal-indexing-api).

        Parameters
        ----------
        corpus_key : CorpusKey
            The unique key identifying the queried corpus.

        request : CreateDocumentRequest

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Document
            Document added to the corpus.

        Examples
        --------
        from vectara import AsyncVectara
        from vectara import StructuredDocument
        from vectara import StructuredDocumentSection
        import asyncio
        client = AsyncVectara(api_key="YOUR_API_KEY", client_id="YOUR_CLIENT_ID", client_secret="YOUR_CLIENT_SECRET", )
        async def main() -> None:
            await client.documents.create(corpus_key='my-corpus-key', request=StructuredDocument(id='my-doc-id', sections=[StructuredDocumentSection(id=1, title='A nice title.', text="I'm a nice document section.", metadata={'section': '1.1'
            }, ), StructuredDocumentSection(id=2, title='Another nice title.', text="I'm another document section on something else.", metadata={'section': '1.2'
            }, )], metadata={'url': 'https://example.com'
            }, ), )
        asyncio.run(main())
        """
        response = await self._raw_client.create(
            corpus_key,
            request=request,
            request_timeout=request_timeout,
            request_timeout_millis=request_timeout_millis,
            request_options=request_options,
        )
        return response.data

    async def get(
        self,
        corpus_key: CorpusKey,
        document_id: str,
        *,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Document:
        """
        Retrieve the content and metadata of a specific document, identified by its
        unique `document_id` from a specific corpus.

        Parameters
        ----------
        corpus_key : CorpusKey
            The unique key identifying the corpus containing the document to retrieve.

        document_id : str
            The document ID of the document to retrieve.
            This `document_id` must be percent encoded.

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Document
            Successfully retrieved the document.

        Examples
        --------
        from vectara import AsyncVectara
        import asyncio
        client = AsyncVectara(api_key="YOUR_API_KEY", client_id="YOUR_CLIENT_ID", client_secret="YOUR_CLIENT_SECRET", )
        async def main() -> None:
            await client.documents.get(corpus_key='my-corpus', document_id='document_id', )
        asyncio.run(main())
        """
        response = await self._raw_client.get(
            corpus_key,
            document_id,
            request_timeout=request_timeout,
            request_timeout_millis=request_timeout_millis,
            request_options=request_options,
        )
        return response.data

    async def delete(
        self,
        corpus_key: CorpusKey,
        document_id: str,
        *,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Permanently delete a document identified by its unique `document_id` from a specific
        corpus. This operation cannot be undone, so use it with caution.

        Parameters
        ----------
        corpus_key : CorpusKey
            The unique key identifying the corpus with the document to delete.

        document_id : str
            The document ID of the document to delete.
            This `document_id` must be percent encoded.

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from vectara import AsyncVectara
        import asyncio
        client = AsyncVectara(api_key="YOUR_API_KEY", client_id="YOUR_CLIENT_ID", client_secret="YOUR_CLIENT_SECRET", )
        async def main() -> None:
            await client.documents.delete(corpus_key='my-corpus', document_id='document_id', )
        asyncio.run(main())
        """
        response = await self._raw_client.delete(
            corpus_key,
            document_id,
            request_timeout=request_timeout,
            request_timeout_millis=request_timeout_millis,
            request_options=request_options,
        )
        return response.data

    async def update(
        self,
        corpus_key: CorpusKey,
        document_id: str,
        *,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Document:
        """
        Updates document identified by its unique `document_id` from a specific
        corpus. The request body metadata is merged with the existing metadata,
        adding or modifying only the specified fields.

        Parameters
        ----------
        corpus_key : CorpusKey
            The unique key identifying the corpus with the document to update.

        document_id : str
            The document ID of the document to update.
            This `document_id` must be percent encoded.

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            The metadata for a document as an arbitrary object. Properties of this object
            can be used by document level filter attributes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Document
            Successfully updated the document.

        Examples
        --------
        from vectara import AsyncVectara
        import asyncio
        client = AsyncVectara(api_key="YOUR_API_KEY", client_id="YOUR_CLIENT_ID", client_secret="YOUR_CLIENT_SECRET", )
        async def main() -> None:
            await client.documents.update(corpus_key='my-corpus', document_id='document_id', )
        asyncio.run(main())
        """
        response = await self._raw_client.update(
            corpus_key,
            document_id,
            request_timeout=request_timeout,
            request_timeout_millis=request_timeout_millis,
            metadata=metadata,
            request_options=request_options,
        )
        return response.data

    async def update_metadata(
        self,
        corpus_key: CorpusKey,
        document_id: str,
        *,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Document:
        """
        Replaces metadata of a document identified by its unique `document_id`
        from a specific corpus.

        Parameters
        ----------
        corpus_key : CorpusKey
            The unique key identifying the corpus with the document to update.

        document_id : str
            The document ID of the document to update.
            This `document_id` must be percent encoded.

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            The metadata for a document as an arbitrary object. Properties of this object
            can be used by document level filter attributes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Document
            Successfully updated the document.

        Examples
        --------
        from vectara import AsyncVectara
        import asyncio
        client = AsyncVectara(api_key="YOUR_API_KEY", client_id="YOUR_CLIENT_ID", client_secret="YOUR_CLIENT_SECRET", )
        async def main() -> None:
            await client.documents.update_metadata(corpus_key='my-corpus', document_id='document_id', )
        asyncio.run(main())
        """
        response = await self._raw_client.update_metadata(
            corpus_key,
            document_id,
            request_timeout=request_timeout,
            request_timeout_millis=request_timeout_millis,
            metadata=metadata,
            request_options=request_options,
        )
        return response.data

    async def summarize(
        self,
        corpus_key: CorpusKey,
        document_id: str,
        *,
        llm_name: str,
        request_timeout: typing.Optional[int] = None,
        request_timeout_millis: typing.Optional[int] = None,
        prompt_template: typing.Optional[str] = OMIT,
        model_parameters: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        stream_response: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SummarizeDocumentResponse:
        """
        Summarize a document identified by its unique `document_id` from a specific corpus.

        Parameters
        ----------
        corpus_key : CorpusKey
            The unique key identifying the corpus containing the document to retrieve.

        document_id : str
            The document ID of the document to retrieve.
            This `document_id` must be percent encoded.

        llm_name : str
            The name of the LLM.

        request_timeout : typing.Optional[int]
            The API will make a best effort to complete the request in the specified seconds or time out.

        request_timeout_millis : typing.Optional[int]
            The API will make a best effort to complete the request in the specified milliseconds or time out.

        prompt_template : typing.Optional[str]
            The prompt template to use when generating the summary.
            Vectara manages both system and user roles and prompts for the generative
            LLM out of the box by default. However, users can override the
            `prompt_template` via this variable. The `prompt_template` is in the form of an
            Apache Velocity template. For more details on how to configure the
            `prompt_template`, see the [long-form documentation](https://docs.vectara.com/docs/prompts/vectara-prompt-engine).

        model_parameters : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Optional parameters for the specified model used when generating the summary.

        stream_response : typing.Optional[bool]
            Indicates whether the response should be streamed or not.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SummarizeDocumentResponse
            Document summarization response on success.

        Examples
        --------
        from vectara import AsyncVectara
        import asyncio
        client = AsyncVectara(api_key="YOUR_API_KEY", client_id="YOUR_CLIENT_ID", client_secret="YOUR_CLIENT_SECRET", )
        async def main() -> None:
            await client.documents.summarize(corpus_key='my-corpus', document_id='document_id', llm_name='llm_name', )
        asyncio.run(main())
        """
        response = await self._raw_client.summarize(
            corpus_key,
            document_id,
            llm_name=llm_name,
            request_timeout=request_timeout,
            request_timeout_millis=request_timeout_millis,
            prompt_template=prompt_template,
            model_parameters=model_parameters,
            stream_response=stream_response,
            request_options=request_options,
        )
        return response.data
